const Database = require('better-sqlite3')
const GenerateSchema = require('generate-schema')
const _ = require('lodash')

function jsonCreateTable(
  data = [],
  { pk = null, tableName = '_job_1', additionalColumns = {} } = {}
) {
  let SQL = `CREATE TABLE IF NOT EXISTS ${tableName} (`
  const {
    items: { properties }
  } = GenerateSchema.json('Job', data)
  const schema = Object.assign(properties, additionalColumns)
  Object.entries(schema).forEach(([key, { type, directive }], index) => {
    let sqlType = 'TEXT'
    if (type === 'string') sqlType = 'TEXT'
    if (type === 'number') sqlType = 'integer'
    if (key === pk) {
      sqlType = `${sqlType} primary key`
    }
    if (directive) {
      sqlType = `${sqlType} ${directive}`
    }
    SQL = `${SQL}${key} ${sqlType} ${
      index !== Object.entries(schema).length - 1 ? ',' : ''
    }`
  })
  return `${SQL});`
}

function insertJsonToTable(
  jsonArray = [],
  { db, tableName = 'tableName', upsert = false } = {}
) {
  let insertStatement = 'insert into'
  if (upsert) {
    insertStatement = 'insert or replace into'
  }
  jsonArray.forEach((data) => {
    const values = Object.values(data)
    const SQL = `${insertStatement} ${tableName} (${Object.keys(data).join(
      ','
    )}) values (${Object.keys(values).fill('?').join(', ')})`
    db.prepare(SQL).run(...values)
  })
}

module.exports = {
  jsonCreateTable,
  insertJsonToTable
}
